# Empaquetado e importación de la librería

<h2> Generación del archivo wheel </h2>

Una vez pasada la parte de implementación se procede a explicar la parte de empaquetado de la librería para su posterior uso. El objetivo del empaquetado es el de lograr un archivo .whl (wheel) que luego podrá ser instalado en cualquier environment para poder importar la librería DISNET en cualquier proyecto. Un archivo wheel es un tipo de extensión diseñada para reemplazar a los antiguos archivos egg. Estos archivos whl facilitan la instalación de paquetes en proyectos Python. En este caso se desea generar un archivo wheel para que luego pueda ser utilizado por el futuro programador en su proyecto instalandolo e importando la librería de DISNET que contendrá.

Previamente se debe asegurar que se respeta la jerarquía de archivos explicada anteriormente y que el archivo setup.py está bien configurado. Es muy importante que cada paquete que contenga la libreria incluya un archivo "__init__.py" vacío, ya que de lo contrario no se podrán localizar los paquetes a pesar de haber sido indicados en el setup y, cuando se vaya a utilizar la librería, dará un error relacionado con que no encuentra el paquete especificado.

Se procede a explicar el procedimiento realizado en un terminal de ubuntu, luego será de manera análoga en otros sistemas unix.

Primero se ha de situar el terminal en la carpeta base del proyecto, es decir en este caso en el directorio DISNET-Base. En esta carpeta deben estar los archivos setup.py, un archivo __init__.py, y el README.md además del paquete tests y el paquete DISNET. Una vez dentro de dicha carpeta se debe ejcutar el siguiente comando:

`$ python setup.py bdist_wheel`

Este comando ejecutará el archivo setup.py y creará el archivo wheel a demas de una serie de carpetas y archivos que no son relevantes para el proposito descrito en esta sección. Este archivo whl mencionado se encontrará en la carpeta DISNET-Base/dist/. El archivo que se genera dentro de la carpeta dst en este ejemplo es el siguiente:

<i> DISNET-0.1-py3-none-any.whl </i>

Los archivos generados como resultado del comando anterior tienen un tipo de nomenclatura especial y por tanto si se cambiase el nombre al archivo generaría fallos a la hora de instalarlo.

<h2> Instalación </h2>

Se va a proceder a explicar el proceso de instalación del paquete whl generado en la seccón anterior. Para ello se van a hacer una serie de suposiciones previas. Una de estas suposiciones será que el sistema donde se va a proceder a instalar dicho paquete tiene un environnement de anaconda  creado con python instalado. El proceso será similar para otras situaciones o gestores de environments.

El primer paso seá el de obtener el archivo wheel, otra suposición que se hará es que se ha descargado de la página web de DISNET y por tanto se encuentra en la carpeta de descargas (Download) en el directorio home (~)

A continuación se deberá abrir un terminal y activar el environment supuesto anteriormente. En esta suposición el environment se llamará ejemplo (como ya se ha comentado en este supuesto se utiliza anaconda pero el proceso sería similar para otros gestores de envs). Para activarlo se utilizará el siguiente comando en el terminal:

`$ conda activate ejemplo`


Posteriormente se deberá mover el terminal a la carpeta donde se encuentra el archivo wheel, en este caso como ya se ha mencionado estará en la ruta "~/Downloads/", y para proceder a ese directorio se utiliza: 

`(ejemplo)$ cd ~/Downloads/`


Por último para poder instalar el paquete wheel en el env se utilizará el comando pip install nombredelarchivo.whl, que en este supuesto será:

`(ejemplo)$ pip install DISNET-0.1-py3-none-any.whl`


Una vez realizados estos pasos se tendrá la librería instalada (junto con la librería requests ya que fue indicado en el setup.py como requisito) en el environment

Podemos desactivar el env seleccionado con:

`$ conda deactivate ejemplo`


<h2> Uso y ejemplos </h2>

Una vez instalado correctamente el archivo wheel se puede proceder a importarlo en el proyecto deseado. Para esto el IDE en el cual se está trabajando se debe haber configurado correctamente el environment utilizado anteriormente, el cual contiene el paquete instalado. 

Una vez configurado el IDE y el proyecto ya se podrá importar la librería DISNET. Para realizar importaciones primero se debe saber el servicio que se desea utilizar, en este caso se va a suponer un programa que va hacer uso del servicio ServicePhenotype:
```python 
import DISNET.services.ServicePhenotype as ServicePhenotype
```

Se ha importado la clase como ServicePhenotype ya que de esta forma no será necesario escribir la ruta cada vez que se use.

Aparte del servicio también se importan los objetos que se desee manipular. Por ejemplo se supondrá que se desea utilizar DiseaseObject, y para su importación:

```python 
import DISNET.objects.DiseaseObject as DiseaseObject
```


Al igual que antes para simplificar su uso se muestra la importación cómo DiseaseObject para evitar repeticiones innecesarias.

Una vez realizados los imports se procede a mostrar el funcionamiento del programa sugerido anteriormente. Este programa consistirá en un método main() que hará una petición a la API para obtener todas las enfermedades. Una vez obtenidas buscará los síntomas asociados a cada una de ellas, mostrando por pantalla dicha información. Para llevar a cabo la tarea descrita se necesitará una instancia del servicio correspondiente (ServicePhenotype) para poder llamar al los métodos requeridos (getAllDiseases() y getSymptomsByDiseaseID()).

```Python
main(self):
    servPheno = ServicePhenotype.ServicePhenotype()
    listDiseases = servPheno.getAllDiseases()
    for disease in listDiseases:
        #para cada disease se puede obtener los sintomas
        print("para la enferemedad " + disease + " se tienen los siguientes sintomas: ")
        print(servPheno.getSymptomsByDiseaseID(disease.getId()))
```

Cuando se trabaja con este tipo de programas se deberá tener en cuenta que pueden existir situaciones en las que una enfermedad no tenga síntomas registrados o que directamente el servidor esté caído y devuelva un error 404. La librería está programada para lanzar en estos casos la excepción NoResultsFound. Será responsabilidad del usuario de la librería cotrolar esta excepción y para ello se puede utilizar un try except de Python de la siguiente manera:

```Python
try:
      …
except NoResultsFoundException:
      print("ERROR: La búsqueda no ha generado resultados")
```



