import unittest
import DISNET.services.ServicePhenotype as ServicePhenotype
import DISNET.services.ServiceBio as ServiceBio
import DISNET.services.ServicePharma as ServicePharma
import DISNET.Exceptions as Exceptions
import DISNET.services.ServiceMapping as ServiceMapping


class Tests(unittest.TestCase):

    def test_ServicePhenotype(self):
        S = ServicePhenotype.ServicePhenotype()
        sources = S.getSources()
        diseases = S.getAllDiseases()
        S.getAllSymptoms()

        self.assertIn("wikipedia",sources, "wikipedia is part of the set of sources")

        source = "wikipedia"
        snapshots = S.getSnapshotsBySource(source)

        with self.assertRaises(Exceptions.NoResultFoundException):
            S.getSnapshotsBySource("NOTASOURCE")

        self.assertIn("2021-01-01", snapshots, "2021-01-01 is part of the set of snapshots")

        snapshot = "2020-01-01"
        diseases = S.getAllDiseases(source, snapshot)
        disease = diseases[0]

        self.assertEqual(disease.getId(), "DIS010790", "the first disease id is DIS010790")

        diseaseList = S.getDiseaseByDiseaseName(disease.getName(), True, source, snapshot)
        disease2 = diseaseList[0]

        self.assertEqual(disease2, disease)

        urls = S.getURLsDisease(disease.getId(), source, snapshot)
        url = urls[0]

        self.assertEqual("http://en.wikipedia.org/wiki/(von_Zumbusch)_acute_generalized_pustular_psoriasis",url)

        symptoms = S.getAllSymptoms(source, snapshot)
        symptom = symptoms[0]

        self.assertEqual(symptom.getId(), "C0013604", "first symptom is C0700129")

        diseaseSymptom = S.getSymptomsByDiseaseID(disease.getId(), source, snapshot)

        self.assertIn(diseaseSymptom[0],symptoms,"disease symptom must be in all symptoms")

        sympname = S.getSymptomsBySymptomName(diseaseSymptom[0].getName(),True,source,snapshot)

        self.assertEqual(diseaseSymptom[0].getId(),sympname[0].getId())

        diseases = S.getDiseasesBySymptomCUI(diseaseSymptom[0].getId())

        meta = S.getMetaData(source,snapshot)

    def test_ServiceBio(self):
        S = ServiceBio.ServiceBio()

        diseases = S.getAllDiseases()
        disease = diseases[0]

        pathways = S.getAllPathways()
        pathway = pathways[0]

        genes = S.getAllGenes()
        gene = genes[0]

        variants = S.getAllVariants()
        variant = variants[0]

        proteins = S.getAllProteins()
        protein = proteins[0]

        ppis = S.getAllPPIS()

        diseasebyName = S.getDiseaseByDiseaseName(disease.getName(),True)
        self.assertEqual(diseasebyName[0],disease)

        diseaseGenes = S.getGenesByDiseaseId(disease.getId())
        self.assertIn(diseaseGenes[0], genes)

        diseaseVariants = S.getVariantsByDiseaseId("C0001175")

        genebyid = S.getGeneByGeneID(gene.getId())
        self.assertEqual(gene,genebyid)

        diseasesbygene = S.getDiseasesByGeneId(gene.getId())
        self.assertIn(diseasesbygene[0],diseases)

        proteinsbygene = S.getProteinsByGeneID(gene.getId())
        self.assertIn(proteinsbygene[0],proteins)

        pathwaybygene = S.getPathwaysByGeneID("1232")
        self.assertIn(pathwaybygene[0],pathways)

        variantsbygene = S.getVariantsByGeneId(72)
        self.assertIn(variantsbygene[0],variants)

        variantbyid = S.getVariantByVariantID(variant.getId())
        self.assertEqual(variantbyid,variant)

        diseasesbyvariant = S.getDiseasesByVariantId(variant.getId())
        self.assertIn(diseasesbyvariant[0], diseases)

        genesbyvariant = S.getGenesByVariantId(variant.getId())
        self.assertIn(genesbyvariant[0],genes)

        pathwaybyid = S.getPathwayByPathwayID(pathway.getId())
        self.assertEqual(pathway,pathwaybyid)

        genesbypathway = S.getGenesByPathwayId(pathway.getId())
        self.assertIn(genesbypathway[0],genes)

        genesbyprotein = S.getGenesByProteinId(protein.getId())
        self.assertIn(genesbyprotein[0],genes)

    def test_ServicePharma(self):
        S = ServicePharma.ServicePharma()

        diseases = S.getAllDiseases()
        disease = diseases[0]

        phenotypes = S.getAllPhenotypes()
        pheno = phenotypes[0]

        drugs = S.getAllDrugs()
        drug = drugs[0]

        targets = S.getAllTargets()
        target = targets[0]

        diseasebyname = S.getDiseaseByDiseaseName(disease.getName(),True)
        self.assertEqual(diseasebyname[0],disease)

        diseasebyid = S.getDiseaseByDiseaseID(disease.getId())
        self.assertEqual(diseasebyid,disease)

        drugsbydisease = S.getDrugsByDiseaseId("C0000786")
        self.assertIn(drugsbydisease[0],drugs)

        phenobyname = S.getPhenotypeByPhenotypeName(pheno.getName(),True)
        self.assertEqual(phenobyname[0],pheno)

        phenobyid = S.getPhenotypeByPhenotypeID(pheno.getId())
        self.assertEqual(phenobyid,pheno)


        drugsbypheno = S.getDrugsByPhenotypeId("C0000731")
        self.assertIn(drugsbypheno[0],drugs)

        drugsbyname = S.getDrugsByDrugName(drug.getName(),True)
        self.assertEqual(drugsbyname[0],drug)

        drugbyid = S.getDrugByDrugID(drug.getId())
        self.assertEqual(drug,drugbyid)

        diseasesbydrug = S.getDiseasesByDrugID("CHEMBL100116")
        self.assertIn(diseasesbydrug[0], diseases)

        phenosbydrug = S.getPhenotypesByDrugsID("CHEMBL99946")
        self.assertIn(phenosbydrug[0], phenotypes)

        targetsbydrug = S.getTargetsByDrugID("CHEMBL42")
        self.assertIn(targetsbydrug[0],targets)

        interactionbydrug = S.getInteractingDrugsByDrugsID("CHEMBL42")
        phenobyinteraction = S.getPhenotypesByDDI("CHEMBL42","CHEMBL1213252")

        targetbyname = S.getTargetByTargetName(target.getName(),True)
        self.assertEqual(targetbyname[0],target)

        targetbyid = S.getTargetByTargetID(target.getId())
        self.assertEqual(targetbyid,target)

        drugsbytarget = S.getDrugsByTargetId("CHEMBL234")
        self.assertIn(drugsbytarget[0],drugs)

    def test_ServiceMapping(self):
        smap = ServiceMapping.ServiceMapping()
        disnetIds = smap.getDisnetIdsByCui("C0000737")
        disnetIdMap = disnetIds[0]
        self.assertEqual(disnetIdMap.getId(),"DIS000005")
        cuis = smap.getCuisByDisnetId("DIS000005")
        cui = cuis[0]
        self.assertEqual(cui.getId(),"C0000737")







if __name__ == '__main__':
    unittest.main()